﻿/*  This script adds various functionality to Presonus Studio One.
    Remove empty tracks, remove disabled tracks, remove inactive track layers,
    custom fade actions like fade at split, and a few track naming and formatting functions.
    
    Author: Lawrence

    Feel free to copy and reuse any of this source code without restriction.

    ** region tags are valid for code folding in Visual Studio Code
    https://code.visualstudio.com/download                                                
*/

// region ----------------------- CONSTANTS -----------------------------------------------------------------------------------------
include_file("functions.js");

const kPackageID = "trackActions";
const scriptTitle = "Track Actions";
var renderFlag = false;
// endregion

function trackAction(actionValue)
{
    this.interfaces = [Host.Interfaces.IEditTask, Host.Interfaces.IParamObserver] //, Host.Interfaces.IObserver]
   
    this.prepareEdit = function (context)
    {
        // create a parameter list for dialog contols
        this.paramList = Host.Classes.createInstance("CCL:ParamList")
        this.paramList.controller = this;

        // edit Box on search form
        this.searchBox = this.paramList.addString("searchBox");

        // help button on search form
        this.helpButton = this.paramList.addInteger(0, 1, "helpButton");

        // edit boxes and replace button on search form
        this.sourceBox = this.paramList.addString("sourceBox");
        this.replaceBox = this.paramList.addString("replaceBox");
        this.buttonReplace = this.paramList.addInteger(0, 1, "buttonReplace");

        // radio buttons for fade options dialog
        this.fadeOptions = this.paramList.addInteger(0, 2, "fadeOptions");

        // length edit box on fade options dialog
        this.fadeLength = this.paramList.addString("fadeLength");

        // save button on fade options dialog
        this.saveFadeOptions = this.paramList.addInteger(0, 1, "saveFadeOptions");
        this.saveFadeOptions.enabled = false;

        // save button on gain options dialog
        this.saveGainOptions = this.paramList.addInteger(0, 1, "saveGainOptions");
        this.saveGainOptions.enabled = false;

        // radio buttons for gain options dialog
        this.gainOptions = this.paramList.addInteger(0, 5, "gainOptions");

			

        return Host.Results.kResultOk;
    }
    
    this.performEdit = function (context)
    {
        // region --------------- CONTEXT VARS --------------------------------------------------------------------------------------

        this.context = context;
        this.functions = context.functions;
        this.functions.executeImmediately = true;

        //  boolean vars for only saving a version backup on the first iteration of a loop
        var savedEmpty = false
        var savedDisabled = false; 
        // endregion
        
        // actions firing targetnig with the actionValue argument
        switch (actionValue)
        {
            case "removeEmptyTracks":

                // get an array of all arrange tracks
                var tracks = getTracks(this.context);

                // iterate tracks and save a version backup before removing the first track
                for (i = 0; i < tracks.length; i++)
                {
                    // get the next track in the array
                    var track = tracks[i];

                    if (track.isEmpty())
                    {
                        // only remove empty visible audio and midi tracks
                        if (track.mediaType == "Audio" || track.mediaType == "Music")
                        {
                            // only save version backup on the first loop iteration
                            if (savedEmpty == false) { saveNewVersion("Before Removing Empty Tracks"); savedEmpty = true; }

                            // remove the track
                            this.functions.removeTrack(track);
                        }
                    }
                }

                break;

            case "removeDisabledTracks":

                // get an array of all arrange tracks
                var tracks = getTracks(this.context);

                // iterate tracks
                for (i = 0; i < tracks.length; i++)
                {
                    // get the next track in the array
                    var track = tracks[i];

                    // avoid errors on tracks that cannot be disabled
                    if (track.channel != undefined && track.channel.disabled == true)
                    {
                        // only save a version backup on first loop iteration
                        if (savedDisabled == false) { saveNewVersion("Before Removing Disabled Tracks"); savedDisabled = true; }

                        // remove the track
                        this.functions.removeTrack(track);
                    }
                }

                break;


            case "removeInactiveLayers":

                // this function cuts media from the active layer, counts the total number of layers, removes layer count -1,
                // deletes any media from the last / active layer, then pastes the original media back onto the active layer

                var trackList = this.context.mainTrackList;
                var mediaWasCut = false;

                // get the first selected track only
                var track = trackList.getSelectedTrack(0);

                // if tno track selected of track class has no layers, do nothing
                if (track == undefined || track.layers.count) { return; }

                // get the total number of layers
                var count = track.layers.count;

                // if the track has no inactive layers, do nothing
                if (count == 1) { return; }

                // save a version backup before removing any layers
                saveNewVersion(track.name + ": Before Removing Layers");

                // isolate / select only the one track to avoid the
                // GUI commands potentially acting on multiple tracks
                // if multiple tracks were originally selected
                trackList.selectTrack(track);

                // force off automataion visibility before selection and cutting
                // otherwise cut may cut and paste automation and not media
                Host.GUI.Commands.interpretCommand("Automation", "Show / Hide", false, Host.Attributes(["State", "0"]));

                // if active layer has any media clips, cut them to the clipboard
                if (!track.isEmpty())
                {
                    Host.GUI.Commands.interpretCommand("Edit", "Select All on Tracks");
                    Host.GUI.Commands.interpretCommand("Edit", "Cut");

                    // flag: media was cut from the active layer
                    mediaWasCut = true;
                }

                // remove all inactive layers, layer count - 1, the last single layer can't be removed
                for (layer = 0; layer < count - 1; layer++)
                {
                    Host.GUI.Commands.deferCommand("Track", "Remove Layer");
                }

                // delete any remaining media on the active last layer              
                Host.GUI.Commands.deferCommand("Edit", "Select All on Tracks");
                Host.GUI.Commands.deferCommand("Edit", "Delete");

                // only fire paste if media was cut from the original active layer to
                // avoiding potentially pasting the wrong data to the last active layer
                if (mediaWasCut)
                {
                    Host.GUI.Commands.deferCommand("Edit", "Paste at Original Position");
                    Host.GUI.Commands.deferCommand("Edit", "Deselect All");
                }

                break;
            
            case "dimMaster":

                // get the master bus object
                var master = getChannels(this.context, 3, false);
                var dimLevel = 1;

                try
                {
                    var data = master.label.split("-");
                    dimLevel = Number("-" + data[1].trim());
                }
                catch (err)
                {
                    // if parse fails, default to -10 dim
                    dimLevel = -10;
                }

                // if the dim level is not a valid number, use the default -10 dim level
                if (isNaN(dimLevel)) { dimLevel = -10 }

                // convert the dim level db value to a floating point number
                var dim = (Math.pow(10, parseFloat(dimLevel / 20)));

                // if the master fader is not at unity set it to unity, else set it to the dim level
                switch (master.volume.toString() != "1")
                {
                    case true:
                        master.volume = 1;
                        break;

                    default:
                        master.volume = dim;
                        break;
                }

            break;
           

            case "resetMixer":

                if (ask("Reset console? This will make a version backup and reset all faders and pans.") == Host.GUI.Constants.kYes)
                {
                    // store a version before the reset
                    saveNewVersion("Before Reset Console");

                    // set the master bus to unity gain
                    var master = getChannels(this.context, 3, false);
                    master.volume = 1;

                    // turn off all plugins, the action toggle assumes they're currently on and the literal
                    // code below to force the state off doesn't work for some reason, still toggles on/off
                    // interpretCommand("Device", "Activate All Inserts", false, Host.Attributes( ["State", "0"] ) );
                    Host.GUI.Commands.interpretCommand("Device", "Activate All Inserts");

                    // get the mixer channel list
                    var environment = this.context.functions.root.environment;
                    var console = environment.find("MixerConsole");
                    var channels = getChannels(this.context, 1, false)

                    // iterate and reset faders and pans for all mixer channels
                    for (i = 0; i < channels.length; i++)
                    {
                        var channel = channels[i];
                        channel.focus();

                        // exclude VCA's from fader reset (uncomment)
                        // if ( channel.toString.indexOf ( "VCA" ) > 0  ) ( continue; )

                        // trap any potential error, fader to infinity
                        if (channel.volume != undefined) { channel.volume = 0; }

                        // VCA's don't have pans, avoid potential error
                        if (channel.pan != undefined) { channel.pan = 0.5; }
                    }
                }

                break;
           
            case "formatNames":

                var tracks = getTracks(this.context);
                for (var i = 0; i < tracks.length; i++)
                {
                    var track = tracks[i];
                    if (track.name == undefined) { continue; }
                    var result = capWords(track.name.trim())
                    this.functions.renameEvent(track, result);
                }

                break;

            case "filterTracks":

                // close the track list to speed it up for larger track counts
                Host.GUI.Commands.interpretCommand("View", "Track List", false, Host.Attributes(["State", "0"]));

                // this.tracklist to pass object var to onParamChange()
                this.trackList = this.context.mainTrackList;
                this.functions = this.context.functions;

                // open the track filter dialog
               return Host.GUI.runDialog(Host.GUI.Themes.getTheme(kPackageID), "TrackSearch", this);

                return Host.Results.kResultOk;

                break;
            
            case "replaceNames":
                // open the track rename dialog
               return Host.GUI.runDialog(Host.GUI.Themes.getTheme(kPackageID), "ReplaceNames", this);
                

                break;

            case "getFadeSettings":

                // get the settings from disk if any
                try
                {
                    var fadeSettings = readTextFile(scriptTitle, "fadeSettings")
                    var data = fadeSettings[0].split("|");
                    this.fadeLength.string = data[0];
                    this.fadeOptions.value = Number(data[1]);
                }
                catch (err)
                {
                    this.fadeLength.value = 0.005;
                    this.fadeOptions.value = 1;
                }

               return Host.GUI.runDialog(Host.GUI.Themes.getTheme(kPackageID), "Fade Settings", this);
                

                break;

            case "fadeAtSplit":

                // get saved fade settings if any and if not return defaults
                var fade = getFadeSettings();

                var clips = new Array();
                var selectFunctions = this.context.editor.createSelectFunctions(this.functions);

                // should be two clips selected, push to array
                var iterator = this.context.iterator
                while (!iterator.done())
                {
                    var event = iterator.next();
                    clips.push(event)
                }

                // types: 0 = linear, Log = 1, Exp = 2
                // fade the clips in and out, right to left
                this.functions.createFadeIn(clips[0], fade.type, fade.length, fade.bend);
                this.functions.createFadeOut(clips[1], fade.type, fade.length, fade.bend);

                // deselect all clips
                Host.GUI.Commands.interpretCommand("Edit", "Deselect All");

                // select rightmost clip again
                selectFunctions.select(clips[0]);

                break;

            case "fadeIn":

                // returns an object with the fade settings in it
                var fade = getFadeSettings();
                var iterator = this.context.iterator

                // apply fade in to all selected audio clips
                while (!iterator.done())
                {
                    var event = iterator.next();
                    this.functions.createFadeIn(event, fade.type, fade.length, fade.bend);
                }

                break;

            case "fadeOut":

                // returns an object with the fade settings in it
                var fade = getFadeSettings();
                var iterator = this.context.iterator

                // apply fade out to all selected audio clips
                while (!iterator.done())
                {
                    var event = iterator.next();
                    this.functions.createFadeOut(event, fade.type, fade.length, fade.bend);
                }

                break;

            case "nudgePlayCursor":
                Host.GUI.Commands.interpretCommand("Edit", "Create Range from Cursor");
                Host.GUI.Commands.interpretCommand("Transport", "Locate Selection End");
                Host.GUI.Commands.interpretCommand("Edit", "Deselect All");
               
                break;

            case "nudgePlayCursorBack":
                Host.studioapp.interpretCommand("Edit", "Create Range from Cursor");
                Host.studioapp.interpretCommand("Edit", "Move Range Back");
                Host.studioapp.interpretCommand("Transport", "Locate Selection");
                Host.studioapp.interpretCommand("Edit", "Deselect All");
                
                break;   

            case "bounceToNewTrack": 
                // couldn't find a way to get the selected track context before the bounce,
                // which changes the selection so it writes the track names to disk
                // and references those to know where to put the clip

                var tracks = getSelectedTracks(this.context)

                if (tracks.length == 1 || tracks.length > 2)
                {
                    renderFlag= false;
                    Host.GUI.alert("This function requires a literal number of two tracks to be selected.");   
                    return ;
                }

                renderFlag = true;
                var path = Host.Url("local://$USERCONTENT/temp/bounceTargets.txt");
                let textFile = Host.IO.createTextFile(path);
                if (textFile)
                {
                    for (i = 0; i < tracks.length; i++)
                    {
                        textFile.writeLine(tracks[i].name);
                    }
                    
                    textFile.close();
                }

                Host.GUI.Commands.interpretCommand("Event", "Bounce To New Track")
            
                break;

            case "finishBounceToNewTrack":

                trackList = this.context.mainTrackList
                var targetName;
              //  var bouncedTrack = getSelectedTracks(this.context)

                var bouncedTrack = trackList.getSelectedTrack(0) ;    
                var sourceTrackName = bouncedTrack.name;

                Host.GUI.Commands.interpretCommand("Edit", "Copy")
                Host.GUI.Commands.interpretCommand("Song", "Remove Track")

                var originalTracks = readTextFile("temp","bounceTargets")
                
        
                for (i=0; i < originalTracks.length; i++)
                {
                    if (originalTracks[i] != sourceTrackName)
                    {
                        targetName = originalTracks[i];
                    }
                }


                for (i = 0; i < trackList.numTracks; i++)
                {
                    var track = trackList.getTrack(i);
                    if (track.name == targetName)
                    {
                        trackList.selectTrack(track);
                        
                    }
                    
                }
             
                Host.GUI.Commands.interpretCommand("Edit", "Paste at Original Position");

                    break;

            case "soloInFront":

                // region --------------- COMMENTS ----------------------------------------------------------------------------------
                /*  This function uses a sub out named Dim Solo to emulate Solo In Front.
                    
                    It leaves most selected mixer channels alone and assigns all other mixer 
                    channels that are assigned to the master bus to the dim solo bus.  That 
                    leaves the selected channels 'in front' of the rest of the music by the
                    amount the dim solo bus is trimmed by with it's fader. 

                    If a selected mixer channel is assigned to a group bus it gets assigned
                    to the master bus so that it won't dim if it's unselected bus channel
                    gets assigned to the dim solo bus.    
                    
                    The master bus turns yellow to indicate Solo In Front mode.
                */
                // endregion
                
                // region --------------- HEADER / PREP -----------------------------------------------------------------------------
                
                // ------------------- SOLO IN FRONT HEADER / PREP ---------------------------------------------------------------------
                
                // load an array of all selected mixer channels
                //var selChannels = getSelectedChannels(this.context)
                var selChannels = getChannels(this.context, 1, true)

                // if no mixer channels are selected, i.e. empty midi track,
                // automation track, etc, then alert the user and exit
                if (selChannels.length == 0)
                {
                    alert ("At least one audio channel must be selected for Solo In Front.");
                    return;
                } 

                // load an array of all sub-out mixer channels
                // var subChannels = getSubChannels(this.context)
                var subChannels = getChannels(this.context, 2, false)
                var dimSoloBus; 

                // try to locate the dim solo bus by it's channel name
                for (i=0; i < subChannels.length; i++)
                {
                    if (subChannels[i].label.trim() == "Dim Solo")
                    {
                        dimSoloBus = subChannels[i];

                        // color the dim solo bus yellow
                        dimSoloBus.findParameter("color").value = getColorVal("ffff66");
                        break;
                    }
                }

                // if the dim solo bus doesn't exist, alert and exit
                if (dimSoloBus == undefined)
                {
                    alert ("This function requires a sub out bus named 'Dim Solo'");
                    return;
                } 
                
                // ----------------------------------------------------------------
                
                // var allChannels = getChannels(this.context)     // array of all ch's
                var allChannels = getChannels(this.context, 1, false)
                var masterBus   = getChannels(this.context, 3, false);  // master bus object
                var toggleOff   = false;                        // bool for toggle

                // ----------------------------------------------------------------

                // go through the channels and if any channel is assigned to 
                // the dim solo bus, set the toggleOff flag true and break
                // toggleOff == true triggers the toggle off code section below

                for (i=0; i < allChannels.length; i++)
                {
                    var channel = allChannels[i];
                    if (channel.getDestinationChannel == undefined)
                        {continue;}

                    var channel = allChannels[i];
                    if ( channel.getDestinationChannel() == dimSoloBus )
                    {
                        toggleOff = true;
                        break;
                    }
                }
                // endregion

                // ---------------------- TOGGLE OFF SOLO IN FRONT ------------------------------------------------------------------

                if (toggleOff)
                {
                    for (i=0; i < allChannels.length; i++)
                    {
                        var channel = allChannels[i];

                        // if the channel is assigned to the dim solo bus
                        // assign it back to the master bus
                        if ( channel.getDestinationChannel() == dimSoloBus )
                        {
                            channel.connectTo(masterBus);
                        }
                    }

                    // get the data from the text file, returns an array of the lines
                    var fileData = readTextFile("temp","soloInFront");

                    // if the array object is valid, proceed
                    if (fileData != undefined)
                    {
                        // reset the original color on the master bus
                        masterBus.findParameter("color").value = Number(fileData[0]);

                        // starting at file data index 1, put any channels that were 
                        // assigned to group busses back into their original busses
                        for (i = 1; i < fileData.length; i++)
                        {
                            var data = fileData[i].split("|");
                            var sourceChannel;
                            var busChannel;

                            if (data.length == 2)   // chan name | bus name
                            {
                                // find the source child channel 
                                for (s=0; s < allChannels.length; s++)
                                {
                                    if (allChannels[s].label == data[0])
                                    {
                                        sourceChannel = allChannels[s];
                                        break;
                                    }
                                }

                                // find the destination group bus or multi-instrument bus channel
                                for (b=0; b < allChannels.length; b++)
                                {
                                    if  (   allChannels[b].label == data[1] && 

                                            (allChannels[b].toString().indexOf("AudioGroup") > 0 ||
                                                allChannels[b].toString().indexOf("SynthBus") > 0    )     
                                        )
                                            {   
                                                busChannel = allChannels[b]; 
                                                break;
                                            }
                                }

                                // assign the channel back to it's original bus channel
                                if (sourceChannel != undefined && busChannel != undefined )
                                {
                                    sourceChannel.connectTo(busChannel);
                                }
                            }
                        }
                    }
                    else
                    {
                        // if the fileData is invalid, reset the bus color 
                        // this shouldn't happen after toggling on SIF but, anyway
                        masterBus.findParameter("color").value = getColorVal("000000");
                    }

                    return;
                }   
                
                // ---------------------- TOGGLE ON SOLO IN FRONT -------------------------------------------------------------------

                if (!toggleOff)
                {   
                    //  the dim solo sub-out from the default I/O setup will be at unity gain 
                    //  for new songs and if the user hasn't changed it yet, set it to -10
                    if ( dimSoloBus.volume == 1 ) { dimSoloBus.volume = dbToFloat(-10) }

                    for (i=0; i < allChannels.length; i++)
                    {
                        var channel = allChannels[i]
                        var destChannel = channel.getDestinationChannel();
                        if (destChannel == undefined)  {continue;}

                        // if the channel is not selected and the channel is assigned
                        // to the master bus, assign it to the dim solo bus
                        if ( selChannels.indexOf(channel) < 0  &&
                                destChannel == masterBus )
                        {
                            channel.connectTo(dimSoloBus);
                        }
                    }

                    /*  if a selected channel is assigned to a group bus, assign it to the 
                        master bus for the solo in front, so that it won't dim with the bus     */

                    // for string building delimited text lines (channel | bus)
                    var busReferences = "";   

                    for (i=0; i < selChannels.length; i++ )
                    {
                        var channel = selChannels[i];
                        var destChannel = channel.getDestinationChannel();
                        if (destChannel == undefined)   {continue;}

                        // don't write master bus connetions to the data file 
                        if (destChannel == masterBus)   {continue;}

                        // if the destination channel is a group bus or multi-instrument bus channel
                        if ( destChannel.toString().indexOf("AudioGroup") > 0 ||
                                destChannel.toString().indexOf("SynthBus") > 0 )
                        {
                            // get the channel name and the bus channel name
                            // string building for data file on loop iteration
                            busReferences = busReferences + channel.label + "|" + 
                            destChannel.label + "\r\n";

                            // assign the channel to the master bus
                            channel.connectTo(masterBus);
                        }
                    }

                    // combine two strings for the text file data
                    // line 1 is the master bus color
                    // subesequent lines are demlimited channel|bus name pairs
                    var fileData = masterBus.findParameter("color").value.toString() +
                    "\r\n" + busReferences;

                    // write the data string to a text file: folder, filename, data
                    writeTextFile("temp", "soloInFront", fileData);
                    
                    // color the master bus yellow to indicate Solo In Front being "on"
                    masterBus.findParameter("color").value = getColorVal("ffff66"); 

                } 
                
                break;

                // ---------------------------------------------------------

                case "renameClipsFromTrack":

                    var iterator = this.context.iterator;
                    var functions = this.context.functions;

                    while (!iterator.done())
                    {
                        var event = iterator.next();
                        trackName = event.getTrack().name;
                        functions.renameEvent(event,trackName);
                    }
                
                    Host.GUI.Commands.interpretCommand("Edit", "Deselect All")

                break;

                // -------------------------------------------------------

                case "renameTracksFromClips":

                    var iterator = this.context.iterator;
                    var functions = this.context.functions;

                    while (!iterator.done())
                    {
                        var event = iterator.next();
                        var track = event.getTrack();
                        var clipName = event.name;
                        functions.renameEvent(track,clipName);
                    }
                
                    Host.GUI.Commands.interpretCommand("Edit", "Deselect All")

                 break;

                // ---------------------------------------------------------

                case "stripNumbers":
                    var functions = this.context.functions;
                    var tracks = getTracks(this.context);

                    for (var i = 0; i < tracks.length; i++) 
                    {  

                        var newname = tracks[i].name.toString().replace(/[0-9]/g, '')
                        var finalname = newname.replace("-", "").trim();
                        functions.renameEvent(tracks[i], finalname);
                    }

                break;

                // ---------------------------------------------------------

                case "trimGain" :
                
                    var gainVal = "-" +  readTextFile(scriptTitle,"gainSettings").toString();
                    if ( gainVal == undefined)  {gainVal = "-3.00"}
                    Host.GUI.Commands.interpretCommand("Audio", "Edit Volume", false, Host.Attributes( ["Level", gainVal + ".00"], ["Relative","1"] ))
                    Host.GUI.Commands.interpretCommand("Audio", "Create Crossfades");

                break;

                // ---------------------------------------------------------

                case "addGain" :

                    var gainVal = readTextFile(scriptTitle,"gainSettings").toString();
                    if ( gainVal == undefined)  {gainVal = "3.00"}      
                    Host.GUI.Commands.interpretCommand("Audio", "Edit Volume", false, Host.Attributes( ["Level", gainVal+".00"], ["Relative","1"] ))
                    Host.GUI.Commands.interpretCommand("Audio", "Create Crossfades");

                break;

                // ---------------------------------------------------------

                case "showGainSettings":

                    // get the setting from disk if any

                    var gainSetting = readTextFile(scriptTitle, "gainSettings");
                   
                    if (gainSetting != undefined)
                    {
                        this.gainOptions.value = Number(gainSetting[0]-1)
                    }
                    else
                    {
                        this.gainOptions.value = 0;
                    }

                    this.saveGainOptions.enabled = false;
                    try
                    {
                      return  Host.GUI.runDialog(Host.GUI.Themes.getTheme(kPackageID), "Gain Settings", this);
                        //return Host.Results.kResultOk;
                    }
                        catch(err)
                    {    }                   

                break;

                case 'centerPlayCursor':

                    Host.GUI.Commands.interpretCommand("Transport", "Set Loop Start")
                    Host.GUI.Commands.interpretCommand("Transport", "Set Loop End")
                    Host.GUI.Commands.interpretCommand("Transport", "Set Loop Start")
                    Host.GUI.Commands.interpretCommand("Transport", "Set Loop End")
                    Host.GUI.Commands.interpretCommand("Transport", "Rewind Bar", false, Host.Attributes( ["Bars","8"] ))
                    Host.GUI.Commands.interpretCommand("Edit", "Create Range from Cursor")
                    Host.GUI.Commands.interpretCommand("Transport", "Forward Bar", false, Host.Attributes( ["Bars","16"] ))
                    Host.GUI.Commands.interpretCommand("Edit", "Range End to Cursor")
                    Host.GUI.Commands.interpretCommand("Zoom", "Zoom to Selection Horizontally", false, Host.Attributes( ["State", "1"]) )
                    Host.GUI.Commands.interpretCommand("Edit", "Deselect All")

                break;

              

        } // end Switch ---------------------------------------------------------------------------------------------------------- <<<<<

    } // <<<<< ------------------- END PERFORM EDIT ------------------------------------------------------------------------------ <<<<<



    // --------------------------- PARAM CHANGED ---------------------------------------------------------------------------------------
    this.paramChanged = function (param)
    {        
        if (param.name == "helpButton")
        {
            alert("'Track | Scene 1' must be set to show all tracks for the filter to work correctly.")
        }

        // enable the save button if length or type changes
        if (param.name == "fadeOptions" || param.name == "fadeLength")
        {
            this.saveFadeOptions.enabled = true;
        }

        // save fade options to disk
        if (param.name == "saveFadeOptions")
        {
            var text = this.fadeLength.string + "|" + this.fadeOptions.value.toString();
            writeTextFile(scriptTitle, "fadeSettings", text)
            this.saveFadeOptions.enabled = false;
        }

        // enable the save button if gain value changes
        if (param.name == "gainOptions" )
        {
            this.saveGainOptions.enabled = true;
        }

        // save gain options to disk
        if (param.name == "saveGainOptions")
        {
            var text =  (this.gainOptions.value + 1).toString();
            writeTextFile(scriptTitle, "gainSettings", text)
            this.saveGainOptions.enabled = false;
        }

        // ----- BATCH REPLACE PARTS OF TRACK NAMES --------------------------------------------------------------

        // replace button on the form
        if (param.name == "buttonReplace")
        {
            var tracks = getTracks(this.context);

            // iterate tracks and perform text replace
            for (i = 0; i < tracks.length; i++)
            {
                var track = tracks[i];
                if (track.name == null) { continue; }
                var name = track.name.toString();
                var newname = name.replace(this.sourceBox.string.trim(), this.replaceBox.string.trim());
                this.functions.renameEvent(track, newname.toString());
            }
        }

        // ----- FILTER TRACKS BY CRITERIA  ----------------------------------------------------------------------

        0
            // turn off to visually speed up
            this.context.editor.showSelection(0);

            // track scene one must make all tracks visible for the filter to work correctly
            // recall track scene 1
            Host.studioapp.interpretCommand("Track", "Select Scene 1");

            // if the search box is empty, all tracks are showing.
            // zoom full to view all tracks on screen and exit
            if (this.searchBox.string.trim() == "")
            {
                Host.studioapp.interpretCommand("Zoom", "Zoom Full", false, Host.Attributes(["State", "1"]));
                return;
            }

            // iterate all tracks, compare, and hide what doesn't match
            for (i = 0; i < this.trackList.numTracks; i++)
            {
                // because track indexes would change if read directly, they're
                // read from an array instead where the indexes remain static
                var track = this.trackList.getTrack(i);

                // exclude vca, folder, and automation tracks from the filter. for example,
                // hiding a folder track would also hide all of it's child tracks
                if (track.toString().indexOf("VCA") > -1 || track.toString().indexOf("Folder") > -1 ||
                    track.toString().indexOf("Automation") > -1) { continue; }

                // account for any potential error with name param
                if (track.name == undefined) { continue; }

                // track has to be independently selected to hide only that track
                this.trackList.selectTrack(track)

                // if it's a media track and it doesn't match the filter, hide it
                if (track.toString().indexOf("MediaTrack") > -1)
                {
                    if (track.name.toUpperCase().indexOf(this.searchBox.string.toUpperCase()) == -1)
                    {
                        Host.studioapp.interpretCommand("Track", "Hide");
                    }
                }
            }

            // zoom full to vertically fill the screen with the visible tracks after the filter
            Host.studioapp.interpretCommand("Zoom", "Zoom Full", false, Host.Attributes(["State", "1"]));

            // restore setting
            this.context.editor.showSelection(1);
        }


    } // end this param chnanged
} /////////////////////////////// END MAIN FUNCTION ///////////////////////////////////////////////////////////////////////////////////

// region ----------------------- CREATE INSTANCES ----------------------------------------------------------------------------------

function removeEmpty() { return new trackAction("removeEmptyTracks"); }
function removeDisabled() { return new trackAction("removeDisabledTracks"); }
function removeLayers() { return new trackAction("removeInactiveLayers"); }
function searchTracks() { return new trackAction("filterTracks"); }
function dimMasterBus() { return new trackAction("dimMaster"); }
function resetConsole() { return new trackAction("resetMixer"); }
function formatNames() { return new trackAction("formatNames"); }
function replaceNames() { return new trackAction("replaceNames"); }
function fadeSettings() { return new trackAction("getFadeSettings"); }
function fadeIn() { return new trackAction("fadeIn"); }
function fadeOut() { return new trackAction("fadeOut"); }
function nudgePlayCursor() { return new trackAction("nudgePlayCursor"); }
function nudgePlayCursorBack() { return new trackAction("nudgePlayCursorBack"); }
function toggleSoloInFront() { return new trackAction("soloInFront"); }

function renameClipsFromTrack() 
{ 
    Host.GUI.Commands.interpretCommand("Edit", "Select All on Tracks")
    return new trackAction("renameClipsFromTrack"); 
}

function renameTracksFromClips()
{
    return new trackAction("renameTracksFromClips");
}

function stripNumbers() { return new trackAction("stripNumbers"); }

function showGainSettings() { return new trackAction("showGainSettings"); }
function trimGain() { Host.GUI.Commands.interpretCommand("Edit", "Split Range"); return new trackAction("trimGain"); }
function addGain() { Host.GUI.Commands.interpretCommand("Edit", "Split Range"); return new trackAction("addGain"); }
function conformTrackColors() { return new trackAction("conformTrackColors"); }
function centerPlayCursor() { return new trackAction("centerPlayCursor"); }

// -------------------------------------------------------------------------------------

/*
    This code is klunky as I could not find a way to parse the tracklist before the bounce
    in the same interface code and also any actions after bounce to new track happen before
    the new is created, so I split it up into different interfaces for force it.  KNowing
    more about the API would certainly allow all of this to be in one action and function
*/

function bounceEventsToTargetTrack()
{
   Host.GUI.Commands.interpretCommand ("Track", "Script Helper: Bounce To New Track")
   if (renderFlag == false) {return;}
   Host.GUI.Commands.interpretCommand ("Track", "Script Helper: Finish Bounce To New Track")
}

function bounceToNewTrack()
{
    return new trackAction("bounceToNewTrack")
}

function finishBounceToNewTrack()
{
    return new trackAction("finishBounceToNewTrack")
}

// -------------------------------------------------------------------------------------

function fadeAtSplit()
{
    Host.GUI.Commands.interpretCommand("Navigation", "Left");
    Host.GUI.Commands.interpretCommand("Navigation", "Right Extend");
    return new trackAction("fadeAtSplit");
}
// endregion

// region ----------------------- HELPER FUNCTIONS ----------------------------------------------------------------------------------


           
// endregion
